using System;
using System.Reflection;
using System.Reflection.Emit;

class ReflectionEmitDemo
{
	static void Main()
	{
		AssemblyName assemblyName = new AssemblyName();
		assemblyName.Name = "DynamicAssembly";

		// Create new assembly
		AssemblyBuilder newAssembly = 
			AppDomain.CurrentDomain.DefineDynamicAssembly(
				assemblyName, AssemblyBuilderAccess.RunAndSave);

		// Create new module in the new assembly
		ModuleBuilder newModule = newAssembly.DefineDynamicModule(
			"NewModule", "EmitedAssembly.exe");

		// Create new type in the new module
		TypeBuilder newType = newModule.DefineType(
			"HelloWorldType", TypeAttributes.Public);

		// Create new method in the new type
		MethodBuilder newMethod = newType.DefineMethod(
			"WriteHello", MethodAttributes.Static | MethodAttributes.Public,
			null, null);

		// Generate the MSIL code
		ILGenerator msilGen = newMethod.GetILGenerator();
		msilGen.EmitWriteLine("Hello World! Today is " + DateTime.Now);
		msilGen.Emit(OpCodes.Ret);

		// Instantiate the new type
		Type helloWorldType = newType.CreateType();
		Object instance = Activator.CreateInstance(helloWorldType);

		// Run the method WriteHello from the new type
		MethodInfo helloWorldMethod = helloWorldType.GetMethod("WriteHello");
		helloWorldMethod.Invoke(instance, null);

		// Save the assembly to executable file
		newAssembly.SetEntryPoint(helloWorldMethod);
		newAssembly.Save("EmitedAssembly.exe");
	}
}
